from django import template
from django.utils.encoding import smart_str
from django.utils.safestring import mark_safe

register = template.Library()


class Nav(object):
    def __init__(self, tree=None, root=None):
        self._root = root or self
        self._tree = tree or {}

    def __getitem__(self, key):
        return Nav(self._tree[key], root=self._root)

    def __str__(self):
        return mark_safe(str(self._text))

    def __bool__(self):
        return bool(self._tree)

    def _get_text(self):
        if hasattr(self._root, "_text_value"):
            return self._root._text_value
        return self._tree

    def _set_text(self, value):
        self._root._text_value = value

    _text = property(_get_text, _set_text)

    def clear(self):
        self._tree = {}

    def update(self, *args, **kwargs):
        self._tree.update(*args, **kwargs)
    
    def get_active_path(self, path=''):
        """Get the dotted path of the active navigation item"""
        for key, value in self._tree.items():
            current_path = path + '.' + key if path else key
            if isinstance(value, dict):
                # Recurse into nested nav
                sub_nav = Nav(value, root=self._root)
                result = sub_nav.get_active_path(current_path)
                if result:
                    return result
            elif value:
                return current_path
        return ''


class NavNode(template.Node):
    def __init__(self, item=None, var_for=None, var_text=None):
        self.item = item
        self.var_name = var_for or "nav"
        self.text = var_text

    def render(self, context):
        first_context_stack = context.dicts[0]
        nav = first_context_stack.get(self.var_name)
        if nav is not context.get(self.var_name):
            raise template.TemplateSyntaxError(
                "'{0}' variable has been altered in current context".format(
                    self.var_name
                )
            )

        if not isinstance(nav, Nav):
            nav = Nav()
            # Copy the stack to avoid leaking into other contexts.
            new_first_context_stack = first_context_stack.copy()
            new_first_context_stack[self.var_name] = nav
            context.dicts[0] = new_first_context_stack

        if self.text:
            nav._text = self.text.resolve(context)
            return ""

        # If self.item was blank then there's nothing else to do here.
        if not self.item:
            return ""

        if nav:
            # If the nav variable is already set, don't do anything.
            return ""

        item = self.item.resolve(context)
        item = item and smart_str(item)
        value = True
        if not item:
            item = ""
        for part in reversed(item.split(".")):
            new_item = {}
            new_item[part] = value
            value = new_item

        nav.clear()
        nav.update(new_item)
        return ""

    def __repr__(self):
        return "<Nav node>"


@register.tag
def nav(parser, token):
    """
    Handles navigation item selection.

    Example usage::

        {# Set the context so {{ nav.home }} (or {{ mynav.home }}) is True #}
        {% nav "home" %} or {% nav "home" for mynav %}

    The most basic (and common) use of the tag is to call ``{% nav [item] %}``,
    where ``[item]`` is the item you want to check is selected.

    By default, this tag creates a ``nav`` context variable. To use an
    alternate context variable name, call ``{% nav [item] for [var_name] %}``.

    Your HTML navigation template should look something like::

        {% block nav %}
        <ul class="nav">
            <li{% if nav.home %} class="selected"{% endif %}>
                <a href="/">Home</a>
            </li>
            <li{% if nav.about %} class="selected"{% endif %}>
                <a href="/about/">About</a>
            </li>
        </ul>
        {% endblock %}

    To override this in a child template, you'd do::

        {% include "base.html" %}
        {% load nav %}

        {% block nav %}
        {% nav "about" %}
        {{ block.super }}
        {% endblock %}

    This works for multiple levels of template inheritance, due to the fact
    that the tag only does anything if the ``nav`` context variable does not
    exist. So only the first ``{% nav %}`` call found will ever be processed.

    As a shortcut, you can use a ``text`` argument and then just reference the
    variable rather than query it with an ``{% if %}`` tag::

        {% nav text ' class="active"' %}
        <ul class="nav">
            <li{{ nav.home }}><a href="/">Home</a></li>
            <li{{ nav.about }}><a href="/about/">About</a></li>
        </ul>

    To create a sub-menu you can check against, simply dot-separate the item::

        {% nav "about_menu.info" %}

    This will be pass for both ``{% if nav.about_menu %}`` and
    ``{% if nav.about_menu.info %}``.
    """
    bits = token.split_contents()

    ok = True
    keys = {"for": False, "text": True}
    node_kwargs = {}
    while len(bits) > 2:
        value = bits.pop()
        key = bits.pop()
        if key not in keys:
            ok = False
            break
        compile_filter = keys.pop(key)
        if compile_filter:
            value = parser.compile_filter(value)
        node_kwargs["var_{0}".format(key)] = value

    if len(bits) > 1:
        # Text argument doesn't expect an item.
        ok = "text" not in node_kwargs
        item = parser.compile_filter(bits[1])
    else:
        item = None

    if not ok:
        raise template.TemplateSyntaxError("Unexpected format for %s tag" % bits[0])

    return NavNode(item, **node_kwargs)


class NavLinkNode(template.Node):
    def __init__(self, nav_item, url_node, nodelist):
        self.nav_item = nav_item
        self.url_node = url_node
        self.nodelist = nodelist

    def render(self, context):
        nav_item = self.nav_item.resolve(context)
        
        # Check if alternate nav variable specified with ':'
        var_name = "nav"
        if ':' in nav_item:
            var_name, nav_item = nav_item.split(':', 1)
        
        nav = context.get(var_name)
        if not isinstance(nav, Nav):
            nav = Nav()
        
        # Check if nav item is active
        try:
            # Get the current active navigation path
            active_path = nav.get_active_path() if nav else ''
            
            # Check if this nav_link matches the active path
            is_exact_match = active_path == nav_item
            is_parent_match = active_path.startswith(nav_item + '.')
            is_link = is_exact_match or is_parent_match
            
            # Get the text value
            nav_text = ''
            if is_link and hasattr(nav, '_text_value') and nav._text_value:
                nav_text = nav._text_value
                if "=" not in nav_text:
                    nav_text = ' class="{}"'.format(nav_text.strip())
        except (KeyError, AttributeError):
            is_link = False
            nav_text = ""
        
        # Get the URL from the url node
        url = self.url_node.render(context)
        
        # Get the content inside the block
        content = self.nodelist.render(context)
        
        # Determine which element to render
        if is_link:
            # Truthy but not exact - render as regular link
            return '<a href="{}"{}>{}</a>'.format(url, nav_text, content)
        else:
            # Falsy - render as span
            return '<span>{}</span>'.format(content)


@register.tag
def navlink(parser, token):
    """
    Renders a link that changes based on navigation state.
    
    Usage::
        
        {% nav text ' class="active"' %}
        {% navlink 'products' 'products:list' %}Products{% endnavlink %}
    
    Renders as:
    - <a href="/products/" class="active">Products</a> - if nav.products is the active item
    - <a href="/products/">Products</a> - if nav.products is a parent item of nav
    - <span>Products</span> - if nav.products doesn't match

    Use {% navlink 'alt_nav:products' ... %} to specify a different nav context.
    """
    from django.template.defaulttags import url

    bits = token.split_contents()
    
    if len(bits) < 3:
        raise template.TemplateSyntaxError(
            "{} tag requires at least two arguments: nav item and url name".format(bits[0])
        )
    
    # First argument is the nav item
    nav_item = parser.compile_filter(bits[1])
    
    # The rest is passed to the url tag
    url_bits = ['url'] + bits[2:]
    url_token = token.__class__(token.token_type, ' '.join(url_bits), token.position, token.lineno)
    
    url_node = url(parser, url_token)
    
    # Parse until endnavlink
    nodelist = parser.parse(('endnavlink',))
    parser.delete_first_token()
    
    return NavLinkNode(nav_item, url_node, nodelist)



from django import template
from django.template.loader import render_to_string
from django.test import TestCase
from django.utils.html import escape

from django_navtag.templatetags.navtag import NavNode

BASIC_TEMPLATE = """
{% load navtag %}
{% nav "banana" %}
{% if nav.apple %}Apple{% endif %}
{% if nav.banana %}Banana{% endif %}
"""

FOR_TEMPLATE = """
{% load navtag %}
{% nav "banana" for othernav %}
{% if othernav.apple %}Apple{% endif %}
{% if othernav.banana %}Banana{% endif %}
"""

NAV_LINK_TEMPLATE = """
{% load navtag %}
{% nav text 'active' %}
{% nav "products" %}
{% navlink 'home' 'home' %}Home{% endnavlink %}
{% navlink 'products' 'products' %}Products{% endnavlink %}
{% navlink 'about' 'about' %}About{% endnavlink %}
"""

NAV_LINK_TEXT_TEMPLATE = """
{% load navtag %}
{% nav text "selected" %}
{% nav "products" %}
{% navlink 'home' 'home' %}Home{% endnavlink %}
{% navlink 'products' 'products' %}Products{% endnavlink %}
{% navlink 'about' 'about' %}About{% endnavlink %}
"""


class NavTagTest(TestCase):
    def test_basic(self):
        t = template.Template(BASIC_TEMPLATE)
        content = t.render(template.Context()).strip()
        self.assertNotIn("Apple", content)
        self.assertIn("Banana", content)

    def test_for(self):
        t = template.Template(FOR_TEMPLATE)
        content = t.render(template.Context()).strip()
        self.assertNotIn("Apple", content)
        self.assertIn("Banana", content)

    def test_basic_extends(self):
        content = render_to_string("navtag_tests/home.txt").strip()
        self.assertIn("- Home (active)", content)
        self.assertNotIn("- Contact (active)", content)

        content = render_to_string("navtag_tests/contact.txt").strip()
        self.assertNotIn("- Home (active)", content)
        self.assertIn("- Contact (active)", content)

    def test_unset(self):
        content = render_to_string("navtag_tests/home.txt").strip()
        self.assertIn("- Home (active)", content)
        self.assertNotIn("- Contact (active)", content)

        content = render_to_string("navtag_tests/home-unset.txt").strip()
        self.assertNotIn("- Home (active)", content)
        self.assertNotIn("- Contact (active)", content)

    def test_heirarchical(self):
        content = render_to_string("navtag_tests/submenu/home.txt").strip()
        self.assertIn("- Home (active)", content)
        self.assertNotIn("- Fruit (active)", content)
        self.assertNotIn("  - Apple (active)", content)
        self.assertNotIn("  - Banana (active)", content)

        content = render_to_string("navtag_tests/submenu/base_fruit.txt").strip()
        self.assertNotIn("- Home (active)", content)
        self.assertIn("- Fruit (active)", content)
        self.assertNotIn("  - Apple (active)", content)
        self.assertNotIn("  - Banana (active)", content)

        content = render_to_string("navtag_tests/submenu/apple.txt").strip()
        self.assertNotIn("- Home (active)", content)
        self.assertIn("- Fruit (active)", content)
        self.assertIn("  - Apple (active)", content)
        self.assertNotIn("  - Banana (active)", content)

        content = render_to_string("navtag_tests/submenu/banana.txt").strip()
        self.assertNotIn("- Home (active)", content)
        self.assertIn("- Fruit (active)", content)
        self.assertNotIn("  - Apple (active)", content)
        self.assertIn("  - Banana (active)", content)

    def test_top_context(self):
        content = render_to_string("navtag_tests/context/home.txt").strip()
        self.assertIn("- Home (active)", content)
        self.assertIn("HOME", content)

    def test_repr(self):
        node = NavNode()
        self.assertEqual(repr(node), "<Nav node>")

    def test_invalid_args(self):
        self.assertRaises(
            template.TemplateSyntaxError,
            template.Template,
            """{% load navtag %}{% nav 'test' unexpected %}""",
        )

    def test_backwards_compatible_empty_tag(self):
        content = template.Template("{% load navtag %}{% nav %}").render(
            template.Context()
        )
        self.assertEqual(content, "")

        content = template.Template("{% load navtag %}{% nav for sidenav %}").render(
            template.Context()
        )
        self.assertEqual(content, "")

    def test_yell_if_context_variable_changed(self):
        t = template.Template('{% load navtag %}{% nav "test" %}{{ nav }}')
        c = template.Context({"nav": "anything"})
        c.update({"nav": "test"})
        self.assertRaises(template.TemplateSyntaxError, t.render, c)

    def test_nav_text(self):
        content = template.Template('{% load navtag %}{% nav text "THIS" %}').render(
            template.Context()
        )
        self.assertEqual(content, "")

    def test_nav_text_none(self):
        content = render_to_string("navtag_tests/text/base.txt").strip()
        self.assertEqual(content, "- Home\n- Contact")

    def test_nav_text_set(self):
        content = render_to_string("navtag_tests/text/home.txt").strip()
        self.assertIn("Home [is active]", content)
        self.assertNotIn("Contact [is active]", content)

        content = render_to_string("navtag_tests/text/contact.txt").strip()
        self.assertNotIn("Home [is active]", content)
        self.assertIn("Contact [is active]", content)

    def test_nav_default_text(self):
        content = (
            template.Template(
                '{% load navtag %}{% nav "fruit" %}{{ nav.fruit }}'
            ).render(template.Context(autoescape=False))
        ).strip()
        self.assertEqual(content, "True")

        content = (
            template.Template(
                '{% load navtag %}{% nav "fruit.banana" %}{{ nav.fruit }}'
            ).render(template.Context(autoescape=False))
        ).strip()
        self.assertEqual(content, "{'banana': True}")

    def test_escaping(self):
        content = (
            template.Template(
                """{% load navtag %}{% nav text ' class="active"' %}"""
                "<p{{ nav }}>{{ name }}</p>"
            ).render(template.Context({"name": "Mc'D"}))
        ).strip()
        escaped = escape("Mc'D")
        self.assertEqual(content, """<p class="active">%s</p>""" % escaped)
    
    def test_navlink_active(self):
        """Test navlink when the nav item is active"""
        # Mock URL resolution
        def mock_url_tag_render(context):
            return "/products/"
        
        t = template.Template(NAV_LINK_TEMPLATE)
        # We need to patch the url rendering since we don't have URL conf
        from django_navtag.templatetags.navtag import NavLinkNode
        original_render = NavLinkNode.render
        
        def patched_render(self, context):
            # Override URL node rendering
            original_url_render = self.url_node.render
            self.url_node.render = lambda ctx: {
                'home': '/home/',
                'products': '/products/', 
                'about': '/about/'
            }.get(self.nav_item.resolve(ctx), '/')
            result = original_render(self, context)
            self.url_node.render = original_url_render
            return result
        
        NavLinkNode.render = patched_render
        try:
            content = t.render(template.Context()).strip()
            # Products should be active with class="active"
            self.assertIn('<a href="/products/" class="active">Products</a>', content)
            # Home and About should be spans since they're not set in nav
            self.assertIn('<span>Home</span>', content)
            self.assertIn('<span>About</span>', content)
        finally:
            NavLinkNode.render = original_render
    
    def test_navlink_inactive(self):
        """Test navlink when no nav item is active"""
        t = template.Template("""
{% load navtag %}
{% navlink 'home' 'home' %}Home{% endnavlink %}
{% navlink 'products' 'products' %}Products{% endnavlink %}
""")
        from django_navtag.templatetags.navtag import NavLinkNode
        original_render = NavLinkNode.render
        
        def patched_render(self, context):
            # Override URL node rendering
            original_url_render = self.url_node.render
            self.url_node.render = lambda ctx: '/' + self.nav_item.resolve(ctx) + '/'
            result = original_render(self, context)
            self.url_node.render = original_url_render
            return result
        
        NavLinkNode.render = patched_render
        try:
            content = t.render(template.Context()).strip()
            # All items should be spans since no nav is set
            self.assertIn('<span>Home</span>', content)
            self.assertIn('<span>Products</span>', content)
            # No links should be present
            self.assertNotIn('<a', content)
        finally:
            NavLinkNode.render = original_render
    
    def test_navlink_with_custom_text(self):
        """Test navlink with custom nav text"""
        from django_navtag.templatetags.navtag import NavLinkNode
        original_render = NavLinkNode.render
        
        def patched_render(self, context):
            # Override URL node rendering
            original_url_render = self.url_node.render
            self.url_node.render = lambda ctx: '/' + self.nav_item.resolve(ctx) + '/'
            result = original_render(self, context)
            self.url_node.render = original_url_render
            return result
        
        NavLinkNode.render = patched_render
        try:
            t = template.Template(NAV_LINK_TEXT_TEMPLATE)
            content = t.render(template.Context()).strip()
            # With nav text "selected", products should have class="selected"
            self.assertIn('<a href="/products/" class="selected">Products</a>', content)
            # Others should be spans since they're not set in nav
            self.assertIn('<span>Home</span>', content)
            self.assertIn('<span>About</span>', content)
            # No items should have class="active" since text is "selected"
            self.assertNotIn('class="active"', content)
        finally:
            NavLinkNode.render = original_render
    
    def test_navlink_hierarchical(self):
        """Test navlink with hierarchical navigation"""
        t = template.Template("""
{% load navtag %}
{% nav text ' class="active"' %}
{% nav "products.electronics" %}
{% navlink 'products' 'products' %}All Products{% endnavlink %}
{% navlink 'products.electronics' 'electronics' %}Electronics{% endnavlink %}
{% navlink 'products.clothing' 'clothing' %}Clothing{% endnavlink %}
""")
        from django_navtag.templatetags.navtag import NavLinkNode
        original_render = NavLinkNode.render
        
        def patched_render(self, context):
            # Override URL node rendering
            original_url_render = self.url_node.render
            self.url_node.render = lambda ctx: '/' + self.nav_item.resolve(ctx).replace('.', '/') + '/'
            result = original_render(self, context)
            self.url_node.render = original_url_render
            return result
        
        NavLinkNode.render = patched_render
        try:
            content = t.render(template.Context()).strip()
            # According to the implementation, both parent and exact match get class="active"
            self.assertIn('<a href="/products/" class="active">All Products</a>', content)
            # products.electronics should be active (exact match)
            self.assertIn('<a href="/products/electronics/" class="active">Electronics</a>', content)
            # products.clothing should be a span
            self.assertIn('<span>Clothing</span>', content)
        finally:
            NavLinkNode.render = original_render
    
    def test_navlink_with_attribute_text(self):
        """Test navlink with nav text containing '=' (e.g., data attributes)"""
        t = template.Template("""
{% load navtag %}
{% nav text ' aria-selected="true"' %}
{% nav "products" %}
{% navlink 'home' 'home' %}Home{% endnavlink %}
{% navlink 'products' 'products' %}Products{% endnavlink %}
""")
        from django_navtag.templatetags.navtag import NavLinkNode
        original_render = NavLinkNode.render
        
        def patched_render(self, context):
            # Override URL node rendering
            original_url_render = self.url_node.render
            self.url_node.render = lambda ctx: '/' + self.nav_item.resolve(ctx) + '/'
            result = original_render(self, context)
            self.url_node.render = original_url_render
            return result
        
        NavLinkNode.render = patched_render
        try:
            content = t.render(template.Context()).strip()
            # Products should have the data attribute
            self.assertIn('<a href="/products/" aria-selected="true">Products</a>', content)
            # Home should be a span
            self.assertIn('<span>Home</span>', content)
            # Should not contain class= since we're using data attributes
            self.assertNotIn('class=', content)
        finally:
            NavLinkNode.render = original_render
